@@ -11,6 +11,8 @@
 #include <unistd.h>
 
 #include "blaze822.h"
+#include "blaze822_priv.h"
+#include "xpledge.h"
 
 static int qflag;
 static int status;
@@ -35,17 +37,17 @@ inc(char *dir)
 	}
 
 	while ((d = readdir(fd))) {
-#if defined(DT_REG) && defined(DT_UNKNOWN)
-		if (d->d_type != DT_REG && d->d_type != DT_UNKNOWN)
+		if (!MAIL_DT(d->d_type))
 			continue;
-#endif
+
 		if (d->d_name[0] == '.')
 			continue;
 
 		snprintf(src, sizeof src, "%s/new/%s",
 		    dir, d->d_name);
 		snprintf(dst, sizeof dst, "%s/cur/%s%s",
-		    dir, d->d_name, strstr(d->d_name, ":2,") ? "" : ":2,");
+		    dir, d->d_name,
+		    strstr(d->d_name, MAILDIR_COLON_SPEC_VER_COMMA) ? "" : MAILDIR_COLON_SPEC_VER_COMMA);
 		if (rename(src, dst) < 0) {
 			fprintf(stderr, "minc: can't rename '%s' to '%s': %s\n",
 			    src, dst, strerror(errno));
@@ -65,20 +67,25 @@ main(int argc, char *argv[])
 {
 	int c, i;
 	while ((c = getopt(argc, argv, "q")) != -1)
-		switch(c) {
+		switch (c) {
 		case 'q': qflag = 1; break;
 		default:
-		usage:
+usage:
 			fprintf(stderr, "Usage: minc [-q] dirs...\n");
 			exit(1);
 		}
 
-	if (optind == argc)
-		goto usage;
+	xpledge("stdio rpath cpath", "");
 
 	status = 0;
-	for (i = optind; i < argc; i++)
-		inc(argv[i]);
+	if (optind == argc) {
+		if (isatty(0))
+			goto usage;
+		blaze822_loop(0, 0, inc);
+	} else {
+		for (i = optind; i < argc; i++)
+			inc(argv[i]);
+	}
 
 	return status;
 }
