@@ -12,9 +12,8 @@
 #include <unistd.h>
 
 #include "blaze822.h"
-
-#define lc(c) ((c) | 0x20)
-#define uc(c) ((c) & 0xdf)
+#include "blaze822_priv.h"
+#include "xpledge.h"
 
 /*
 
@@ -49,34 +48,47 @@ static long tdirs;
 static long tunseen;
 static long tflagged;
 static long tcount;
+static long tmatched;
 
 void
 list(char *prefix, char *file)
 {
+	char *f = 0;
+
+	if (flagset || iflag) {
+		size_t prefixlen;
+
+		f = strstr(file, MAILDIR_COLON_SPEC_VER_COMMA);
+
+		if (!f &&
+		    prefix &&
+		    (prefixlen = strlen(prefix)) &&
+		    prefixlen >= 4 &&
+		    strcmp(prefix + prefixlen - 4, "/new") == 0)
+			f = MAILDIR_COLON_SPEC_VER_COMMA;
+	}
+
 	if (flagset) {
 		int sum = 0;
-		char *f = strstr(file, ":2,");
 		if (!f)
 			return;
 		icount++;
 		tcount++;
-		f += 3;
-		while (*f) {
-			if (flags[(unsigned int)*f] == -1)
+		char *g;
+		for (g = f + 3; *g; g++)
+			if (flags[(unsigned int)*g] == -1)
 				return;
-			if (flags[(unsigned int)*f] == 1)
+			else if (flags[(unsigned int)*g] == 1)
 				sum++;
-			f++;
-		}
 		if (sum != flagsum)
 			return;
 	}
 
 	if (iflag) {
-		char *f = strstr(file, ":2,");
 		if (!f)
 			return;
 		imatched++;
+		tmatched++;
 		if (!flagset)
 			icount++, tcount++;
 		if (!strchr(f, 'S'))
@@ -99,8 +111,8 @@ list(char *prefix, char *file)
 #include <sys/syscall.h>
 
 struct linux_dirent64 {
-	ino64_t d_ino;           /* 64-bit inode number */
-	off64_t d_off;           /* 64-bit offset to next structure */
+	uint64_t d_ino;          /* 64-bit inode number */
+	int64_t d_off;           /* 64-bit offset to next structure */
 	unsigned short d_reclen; /* Size of this dirent */
 	unsigned char d_type;    /* File type */
 	char d_name[];           /* Filename (null-terminated) */
@@ -134,7 +146,7 @@ listdir(char *dir)
 
 		for (bpos = 0; bpos < nread; bpos += d->d_reclen) {
 			d = (struct linux_dirent64 *)(buf + bpos);
-			if (d->d_type != DT_REG && d->d_type != DT_UNKNOWN)
+			if (!MAIL_DT(d->d_type))
 				continue;
 			if (d->d_name[0] == '.')
 				continue;
@@ -155,7 +167,7 @@ listdir(char *dir)
 	if (!fd)
 		return;
 	while ((d = readdir(fd))) {
-		if (d->d_type != DT_REG && d->d_type != DT_UNKNOWN)
+		if (!MAIL_DT(d->d_type))
 			continue;
 		if (d->d_name[0] == '.')
 			continue;
@@ -206,8 +218,12 @@ listarg(char *arg)
 
 		if (iflag && (imatched || (maildir && !flagset))) {
 			tdirs++;
-			printf("%6ld unseen  %3ld flagged  %6ld msg  %s\n",
-			    iunseen, iflagged, icount, arg);
+			if (flagset && imatched)
+				printf("%6ld matched  %6ld unseen  %3ld flagged  %6ld msg  %s\n",
+				    imatched, iunseen, iflagged, icount, arg);
+			else
+				printf("%6ld unseen  %3ld flagged  %6ld msg  %s\n",
+				    iunseen, iflagged, icount, arg);
 		}
 
 		icount = gcount;
@@ -257,6 +273,8 @@ usage:
 
 	int i;
 
+	xpledge("stdio rpath", "");
+
 	for (i = 0, flagsum = 0, flagset = 0; (size_t)i < sizeof flags; i++) {
 		if (flags[i] != 0)
 			flagset++;
@@ -273,9 +291,14 @@ usage:
 			listarg(argv[i]);
 	}
 
-	if (iflag && tdirs > 1)
-		printf("%6ld unseen  %3ld flagged  %6ld msg\n",
-		    tunseen, tflagged, tcount);
+	if (iflag && tdirs > 1) {
+		if (flagset)
+			printf("%6ld matched  %6ld unseen  %3ld flagged  %6ld msg\n",
+			    tmatched, tunseen, tflagged, tcount);
+		else
+			printf("%6ld unseen  %3ld flagged  %6ld msg\n",
+			    tunseen, tflagged, tcount);
+	}
 
 	return 0;
 }
