@@ -1,11 +1,17 @@
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 700
 #endif
+#ifdef __sun
+#define __EXTENSIONS__ /* to get TIOCGWINSZ */
+#endif
+
+#include "xpledge.h"
 
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <locale.h>
@@ -16,6 +22,7 @@
 #include <time.h>
 #include <unistd.h>
 #include <wchar.h>
+#include <termios.h>
 
 #include "blaze822.h"
 #include "u8decode.h"
@@ -29,6 +36,7 @@ static int alias_idx;
 
 static int Iflag;
 static int nflag;
+static int vflag;
 static int curyear;
 static time_t now;
 static char default_fflag[] = "%c%u%r %-3n %10d %17f %t %2i%s";
@@ -52,9 +60,13 @@ u8putstr(FILE *out, char *s, ssize_t l, int pad)
 			int r = u8decode(s, &c);
 			if (r < 0) {
 				r = 1;
+				l--;
 				fprintf(out, "%lc", (wint_t)replacement);
 			} else {
-				l -= wcwidth((wchar_t)c);
+				int w = wcwidth((wchar_t)c);
+				if (w < 0)
+					w = 2;   /* assume worst width */
+				l -= w;
 				if (l >= 0)
 					fwrite(s, 1, r, out);
 			}
@@ -133,6 +145,33 @@ fmt_date(struct message *msg, int w, int iso)
 			strftime(date, sizeof date, "%Y-%m-%d %H:%M", tm);
 		else
 			strftime(date, sizeof date, "%Y-%m-%d", tm);
+	} else if (w >= 19) {
+		// https://dotti.me/
+
+		tm = gmtime(&t);
+		strftime(date, sizeof date, "%Y-%m-%dT%H\xc2\xb7%M", tm);
+		char *d = date + strlen(date);
+
+		char *e = v + strlen(v);
+		while (e > v && (*e != '+' && *e != '-'))
+			e--;
+
+		if (isdigit(e[1]) && isdigit(e[2]) && isdigit(e[3]) && isdigit(e[4])) {
+			*d++ = e[0];
+			*d++ = e[1];
+			*d++ = e[2];
+			if (e[3] != '0' || e[4] != '0') {
+				*d++ = ':';
+				*d++ = e[3];
+				*d++ = e[4];
+			}
+			*d = 0;
+		} else {
+			*d++ = '+';
+			*d++ = '0';
+			*d++ = '0';
+			*d = 0;
+		}
 	} else if (w < 10) {
 		if (tm->tm_year != curyear)
 			strftime(date, sizeof date, "%b%y", tm);
@@ -283,7 +322,7 @@ oneline(char *file)
 	}
 
 	struct message *msg = blaze822(file);
-	char *flags = msg ? strstr(file, ":2,") : 0;
+	char *flags = msg ? strstr(file, MAILDIR_COLON_SPEC_VER_COMMA) : 0;
 	if (!flags)
 		flags = "";
 	else
@@ -338,12 +377,12 @@ oneline(char *file)
 			wleft--;
 			break;
 		case 'u':  // unseen
-			if (strchr(flags, 'F'))
+			if (strchr(flags, 'T'))
+				putchar('x');
+			else if (strchr(flags, 'F'))
 				putchar('*');
 			else if (msg && !strchr(flags, 'S'))
 				putchar('.');
-			else if (strchr(flags, 'T'))
-				putchar('x');
 			else
 				putchar(' ');
 			wleft--;
@@ -432,13 +471,21 @@ oneline(char *file)
 		case 'F':
 			{
 				char *e = file + strlen(file);
+
+				if (!msg)
+					goto empty;
+
 				while (file < e && *e != '/')
 					e--;
+				if (file == e)
+					goto empty;
 				e--;
 				while (file < e && *e != '/')
 					e--;
 				while (file < e && *e == '/')
 					e--;
+				if (file == e)
+					goto empty;
 				char *b = e;
 				e++;
 				while (file < b && *b != '/')
@@ -447,6 +494,11 @@ oneline(char *file)
 					b++;
 				if (*b == '.')
 					b++;
+
+				if (0) {
+empty:
+					b = e = "";
+				}
 				if (w) {
 					if (w < 0)
 						w = -w;
@@ -489,19 +541,22 @@ oneline(char *file)
 int
 main(int argc, char *argv[])
 {
-	pid_t pid1 = -1;
+	pid_t pager_pid = -1;
 
 	int c;
-	while ((c = getopt(argc, argv, "If:n")) != -1)
+	while ((c = getopt(argc, argv, "If:nv")) != -1)
 		switch (c) {
 		case 'I': Iflag++; break;
 		case 'f': fflag = optarg; break;
 		case 'n': nflag = 1; break;
+		case 'v': vflag = 1; break;
 		default:
-			fprintf(stderr, "Usage: mscan [-n] [-f format] [-I] [msgs...]\n");
+			fprintf(stderr, "Usage: mscan [-Inv] [-f format] [msgs...]\n");
 			exit(1);
 		}
 
+	xpledge("stdio rpath tty proc exec", NULL);
+
 	if (nflag) {
 		if (argc == optind && isatty(0))
 			blaze822_loop1(":", numline);
@@ -520,23 +575,28 @@ main(int argc, char *argv[])
 
 	struct winsize w;
 	int ttyfd = open("/dev/tty", O_RDONLY | O_NOCTTY);
-	if (ttyfd >= 0 && ioctl(ttyfd, TIOCGWINSZ, &w) == 0) {
-		cols = w.ws_col;
+	if (ttyfd >= 0) {
+		if (ioctl(ttyfd, TIOCGWINSZ, &w) == 0)
+			cols = w.ws_col;
+		close(ttyfd);
+	}
 
+	if (isatty(1)) {
 		char *pg;
 		pg = getenv("MBLAZE_PAGER");
 		if (!pg)
 			pg = getenv("PAGER");
 		if (pg && *pg && strcmp(pg, "cat") != 0) {
-			pid1 = pipeto(pg);
-			if (pid1 < 0)
+			pager_pid = pipeto(pg);
+			if (pager_pid < 0)
 				fprintf(stderr,
 				    "mscan: spawning pager '%s': %s\n",
 				    pg, strerror(errno));
 		}
 	}
-	if (ttyfd >= 0)
-		close(ttyfd);
+
+	xpledge("stdio rpath", "");
+
 	if (getenv("COLUMNS"))
 		cols = atoi(getenv("COLUMNS"));
 	if (cols <= 40)
@@ -567,10 +627,12 @@ main(int argc, char *argv[])
 		i = blaze822_loop1(":", oneline);
 	else
 		i = blaze822_loop(argc-optind, argv+optind, oneline);
-	fprintf(stderr, "%ld mails scanned\n", i);
 
-	if (pid1 > 0)
-		pipeclose(pid1);
+	if (pager_pid > 0)
+		pipeclose(pager_pid);
+
+	if (vflag)
+		fprintf(stderr, "%ld mails scanned\n", i);
 
 	return 0;
 }
