@@ -10,6 +10,7 @@
 #include <unistd.h>
 
 #include "blaze822.h"
+#include "xpledge.h"
 
 struct mail {
 	char *file;
@@ -218,8 +219,8 @@ unreadorder(const void *a, const void *b)
 	struct mail *ia = (struct mail *)a;
 	struct mail *ib = (struct mail *)b;
 
-	char *fa = strstr(ia->file, ":2,");
-	char *fb = strstr(ib->file, ":2,");
+	char *fa = strstr(ia->file, MAILDIR_COLON_SPEC_VER_COMMA);
+	char *fb = strstr(ib->file, MAILDIR_COLON_SPEC_VER_COMMA);
 
 	int unreada = fa ? !strchr(fa, 'S') : 0;
 	int unreadb = fb ? !strchr(fb, 'S') : 0;
@@ -233,8 +234,8 @@ flaggedorder(const void *a, const void *b)
 	struct mail *ia = (struct mail *)a;
 	struct mail *ib = (struct mail *)b;
 
-	char *fa = strstr(ia->file, ":2,");
-	char *fb = strstr(ib->file, ":2,");
+	char *fa = strstr(ia->file, MAILDIR_COLON_SPEC_VER_COMMA);
+	char *fb = strstr(ib->file, MAILDIR_COLON_SPEC_VER_COMMA);
 
 	int unreada = fa ? !!strchr(fa, 'F') : 0;
 	int unreadb = fb ? !!strchr(fb, 'F') : 0;
@@ -278,9 +279,9 @@ add(char *file)
 int
 order(const void *a, const void *b)
 {
-	int i, r;
-	for (i = 0, r = 0; i < order_idx; i++) {
-		r = (sortorders[i])(a, b);
+	int i;
+	for (i = 0; i < order_idx; i++) {
+		int r = (sortorders[i])(a, b);
 		if (r != 0)
 			return r;
 	}
@@ -316,8 +317,9 @@ main(int argc, char *argv[])
 			exit(1);
 		}
 
+	xpledge("stdio rpath", "");
 
-	mails = calloc(sizeof (struct mail), mailalloc);
+	mails = calloc(mailalloc, sizeof (struct mail));
 	if (!mails)
 		exit(-1);
 
