@@ -19,14 +19,16 @@
 #include <unistd.h>
 
 #include "blaze822.h"
+#include "xpledge.h"
 
 static int vflag;
+static int pflag;
+static int rflag;
 static int optional;
 
 struct container {
 	char *mid;
 	char *file;
-	struct message *msg;
 	time_t date;
 	struct container *parent;
 	struct container *child;
@@ -64,7 +66,7 @@ mid(struct message *msg)
 	} else {
 		// invent new message-id for internal tracking
 		static long i;
-		char buf[32];
+		char buf[40];
 		snprintf(buf, sizeof buf, "thread%08ld@localhost", ++i);
 		return strdup(buf);
 	}
@@ -74,13 +76,16 @@ struct container *
 midcont(char *mid)
 {
 	struct container key, **result;
+	if (!mid)
+		exit(111);
 	key.mid = mid;
 
 	if (!(result = tfind(&key, &mids, midorder))) {
 		struct container *c = malloc(sizeof (struct container));
+		if (!c)
+			exit(111);
 		c->mid = mid;
 		c->file = 0;
-		c->msg = 0;
 		c->date = -1;
 		c->optional = 0;
 		c->parent = c->child = c->next = 0;
@@ -97,7 +102,6 @@ store_id(char *file, struct message *msg)
 
 	c = midcont(mid(msg));
 	c->file = strdup(file);
-	c->msg = msg;
 	c->optional = optional;
 
 	return c;
@@ -232,6 +236,8 @@ out2:
 			c->child = 0;
 		}
 	}
+
+	blaze822_free(msg);
 }
 
 time_t
@@ -281,7 +287,8 @@ void
 find_roots()
 {
 	top = malloc(sizeof (struct container));
-	top->msg = 0;
+	if (!top)
+		exit(111);
 	top->date = -1;
 	top->file = 0;
 	top->next = top->child = top->parent = 0;
@@ -306,8 +313,8 @@ prune_tree(struct container *c, int depth)
 			// turn into child if we don't exist and only have a child
 			c->mid = c->child->mid;
 			c->file = c->child->file;
-			c->msg = c->child->msg;
-			c->date = c->child->date;
+			if (c->child->date > c->date)
+				c->date = c->child->date;
 			c->optional = c->child->optional;
 			c->child = c->child->child;
 		}
@@ -340,6 +347,12 @@ dateorder(const void *a, const void *b)
 	return 0;
 }
 
+static int
+reverse_dateorder(const void *a, const void *b)
+{
+	return dateorder(b, a);
+}
+
 void
 sort_tree(struct container *c, int depth)
 {
@@ -352,14 +365,15 @@ sort_tree(struct container *c, int depth)
 		if (i == 1)  // no sort needed
 			return;
 
-		struct container **a = calloc(sizeof (struct container *), i);
+		struct container **a = calloc(i, sizeof (struct container *));
 		if (!a)
 			return;
 
 		for (r = c->child, i = 0; r; r = r->next, i++)
 			a[i] = r;
 
-		qsort(a, i, sizeof (struct container *), dateorder);
+		qsort(a, i, sizeof (struct container *),
+		    rflag && depth < 0 ? reverse_dateorder : dateorder);
 
 		c->child = a[0];
 		for (j = 0; j+1 < i; j++)
@@ -375,7 +389,8 @@ print_tree(struct container *c, int depth)
 {
 	do {
 		// skip toplevel threads when they are unresolved or all optional
-		if (depth <= 1 &&
+		// (or when -p is given, skip those subthreads)
+		if ((depth <= 1 || pflag) &&
 		    (c->optional || !c->file) &&
 		    (!c->child || alloptional(c->child)))
 			continue;
@@ -399,25 +414,31 @@ int
 main(int argc, char *argv[])
 {
 	int c;
-	long i;
 
 	optional = 1;
 
-	while ((c = getopt(argc, argv, "S:v")) != -1)
+	xpledge("stdio rpath", "");
+
+	while ((c = getopt(argc, argv, "S:prv")) != -1)
 		switch (c) {
 		case 'S': blaze822_loop1(optarg, thread); break;
 		case 'v': vflag = 1; break;
+		case 'p': pflag = 1; break;
+		case 'r': rflag = 1; break;
 		default:
-			fprintf(stderr, "Usage: mthread [-v] [-S dir] [msgs...]\n");
+			fprintf(stderr, "Usage: mthread [-vpr] [-S dir] [msgs...]\n");
 			exit(1);
 		}
 
 	optional = 0;
 
 	if (argc == optind && isatty(0))
-		i = blaze822_loop1(":", thread);
+		blaze822_loop1(":", thread);
 	else
-		i = blaze822_loop(argc-optind, argv+optind, thread);
+		blaze822_loop(argc-optind, argv+optind, thread);
+
+	// the tree of all toplevel threads has depth -1,
+	// so toplevel threads have depth 0.
 
 	find_roots();
 	if (!vflag)
@@ -425,7 +446,5 @@ main(int argc, char *argv[])
 	sort_tree(top, -1);
 	print_tree(top, -1);
 
-	fprintf(stderr, "%ld mails threaded\n", i);
-
 	return 0;
 }
